package lab2;

import  lab2.Exceptions.*;

/**
 * Classe ArrayStack:
 *
 * Realizzazione della pila mediante array, utilizzando la specifica
 * di Java Collections API disponibile in <code>java.util.*</code>.
 * @see lab2.Stack
 * @version 2.0 Febbraio 2001
 * @author A. Brogi      (<A HREF="mailto:brogi@di.unipi.it">brogi@di.unipi.it</A>)
 * @author A. Corradini  (<A HREF="mailto:andrea@di.unipi.it">andrea@di.unipi.it</A>)
 * @author F. Gadducci   (<A HREF="mailto:gadducci@di.unipi.it">gadducci@di.unipi.it</A>)
 * @author R. Grossi     (<A HREF="mailto:grossi@di.unipi.it">grossi@di.unipi.it</A>)
 * @author C. Montangero (<A HREF="mailto:monta@di.unipi.it">monta@di.unipi.it</A>)
 * @author F. Romani     (<A HREF="mailto:romani@di.unipi.it">romani@di.unipi.it</A>)
 * @author L. Semini     (<A HREF="mailto:semini@di.unipi.it">semini@di.unipi.it</A>)
 */
public class ArrayStack implements Stack {
    static final int DEFAULT_SIZE = 32;
    
    protected Object [ ] theArray;
    protected int        top;
    
    /**
     * Costruttore della pila.
     */
    public ArrayStack( ) {
        theArray = new Object[ DEFAULT_SIZE ];
        top = -1;
    }

    /**
     * Verifica che la pila sia logicamente vuota.
     * @return  <code>true</code> se la pila &egrave; vuota;
     *          <code>false</code> altrimenti.
     */
    public boolean isEmpty( ) {
        return top == -1;
    }

    /**
     *  Svuota la pila. 
     */
    public void clear( ) {
        top = -1;
    }

    /**
     * Restituisce l'oggetto in cima alla pila senza estrarlo.
     * @return   l'oggetto in cima.
     * @exception EmptyStackException con pila vuota.
     */
    public Object  peek( ) {
        if ( isEmpty( ) )
            throw new EmptyStackException( );
        return theArray[ top ];
    }

    /**
     * Inserisce un oggetto in cima alla pila.
     * @param x  l'oggetto da inserire.
     * @return   l'oggetto inserito.
     * @exception IllegalArgumentException se l'argomento passato
     *            &egrave; <code>null</code>.  
     */
    public Object push( Object x ) {
	if ( x == null )
	    throw new IllegalArgumentException( );
        if ( top == theArray.length - 1 )
            doubleArray( );
        return theArray[ ++top ] = x;
    }

    /**
     * Rimuove e restituisce l'oggetto in cima alla pila.
     * @return   l'oggetto in cima.
     * @exception EmptyStackException con pila vuota.
     */
    public Object pop( ) {
        if ( isEmpty( ) )
            throw new EmptyStackException( );
        return theArray[ top-- ];
    }

    // Metodo interno per raddoppiare la dimensione di theArray.
    protected void doubleArray( ) {
        Object [ ] newArray;

        newArray = new Object[ theArray.length * 2 ];
        for ( int i = 0; i < theArray.length; i++ )
              newArray[ i ] = theArray[ i ];
        theArray = newArray;
    }
}
