package lab2;

/** 
 * Classe BinNode:
 *
 * Classe usata per descrivere il singolo nodo di un albero binario.
 * &Egrave; possibile creare nodi con riferimenti nulli al padre e/o a
 * entrambi i figli in base al <em>numero</em> di argomenti del
 * costruttore.  La classe viene usata per l'implementazione mediante
 * alberi binari dei tipi di dato astratto (il tipo di dato astratto
 * "insieme ordinato" realizzato mediante alberi binari di ricerca
 * &egrave; diverso da BinNode, che pu&ograve essere usata o meno nella
 * sua implementazione).
 * @version 2.0 Febbraio 2001
 * @author A. Brogi      (<A HREF="mailto:brogi@di.unipi.it">brogi@di.unipi.it</A>)
 * @author A. Corradini  (<A HREF="mailto:andrea@di.unipi.it">andrea@di.unipi.it</A>)
 * @author F. Gadducci   (<A HREF="mailto:gadducci@di.unipi.it">gadducci@di.unipi.it</A>)
 * @author R. Grossi     (<A HREF="mailto:grossi@di.unipi.it">grossi@di.unipi.it</A>)
 * @author C. Montangero (<A HREF="mailto:monta@di.unipi.it">monta@di.unipi.it</A>)
 * @author F. Romani     (<A HREF="mailto:romani@di.unipi.it">romani@di.unipi.it</A>)
 * @author L. Semini     (<A HREF="mailto:semini@di.unipi.it">semini@di.unipi.it</A>)
 */
public class BinNode {    
    public Object  key;      // Campo chiave
    public BinNode left;     // Riferimento al figlio sinistro
    public BinNode right;    // Riferimento al figlio destro
    public BinNode parent;   // Riferimento al padre
    
    /*
      Costruttore di un BinNode in cui l'oggetto k (diverso da null)
      e' il campo chiave, i parametri l e r sono riferimenti al figlio
      sinistro e destro, rispettivamente, e il parametro p e' il
      riferimento al padre.  
    */
    public BinNode( Object k, BinNode l, BinNode r, BinNode p ) {
	if ( k == null )
	    throw new IllegalArgumentException( );
	key = k;
	left = l;
	right = r;
	parent = p;
	// Aggiorna il campo padre del figlio sinistro (se esiste)
	if (l != null) 
	    l.parent = this;
	// Aggiorna il campo padre del figlio destro (se esiste)
	if (r != null) 
	    r.parent = this;
    }

    // Varianti del costruttore suddetto
    public BinNode( Object k ) {
	this (k, null, null, null);
    }
    
    public BinNode( Object k, BinNode p) {
	this (k, null, null, p);
    }
    
    public BinNode( Object k, BinNode l, BinNode r ) {
	this (k, l, r, null);
    }
        
}
