package lab2;

/**
 * Interfaccia Iterator:
 *
 * Specifica di iteratore nelle Java Collections API, riportata
 * verbatim.  Definisce implicitamente un cursore, posizionato prima
 * dell'oggetto da esaminare inizialmente e quindi spostato
 * sull'oggetto successivo a ogni chiamata di <code>next</code>,
 * fintanto che ci sono oggetti da esaminare.  Tale situazione
 * &egrave; segnalata da <code>hasNext</code>.  Si veda <A
 * HREF="http://java.sun.com/products/jdk/1.2/docs/guide/collections/reference.html">
 * <TT>java.util.*</TT> (JDK 1.2)</A> [Iterator].
 *
 * <pre>
 * ****************************** METODI PUBBLICI ***************************
 * boolean hasNext( )  --> Verifica se ci sono ulteriori oggetti da esaminare
 * Object next( )      --> Restituisce il prossimo oggetto da esaminare
 * void remove( )      --> Cancella l'ultimo oggetto restituito da next
 * ****************************** ECCEZIONI *********************************
 * next genera NoSuchElementException se invocata quando hasNet e' falsa;
 * remove genera IllegalStateException se invocata impropriamente;
 * next e remove generano ConcurrentModificationException se avviene
 * una modifica con altri metodi.
 * </pre>
 * @version 2.0 Febbraio 2001 */
public interface Iterator {
    /**
     * Indica se ci sono ulteriori elementi da esaminare nell'iterazione
     * corrente. 
     * @return <tt>true</tt> se ci sono ulteriori elementi;
     *         <tt>false</tt> altrimenti.
     * @exception ConcurrentModificationException se avviene
     *            una modifica concorrente degli stessi oggetti
     *            mediante qualche altro metodo.
     */
    boolean hasNext( );

    /**
     * Restituisce l'oggetto successivo da esaminare nell'iterazione.
     * @return il prossimo oggetto da esaminare.
     * @exception NoSuchElementException se non ci
     * sono altri oggetti da esaminare.  
     * @exception ConcurrentModificationException se avviene una
     * modifica concorrente degli stessi oggetti mediante qualche
     * altro metodo.  
     */
    Object next( );

    /**
     * Cancella l'oggetto restituito dall'ultima chiamata al metodo
     * <code>next</code>.  Il metodo pu&ograve; essere invocato solo
     * una volta per ciascuna chiamata a <code>next</code>. Il
     * comportamento &egrave; arbitrario in caso di modifica
     * concorrente degli stessi oggetti, mediante qualche altro metodo.
     * @exception IllegalStateException se nessuna chiamata al metodo
     * <tt>next</tt> &egrave; stata precedentemente effettuata, oppure
     * una chiamata a <tt>remove</tt> &egrave; stata gi&agrave;
     * eseguita in corrispondenza della stessa chiamata a
     * <tt>next</tt>.
     * @exception ConcurrentModificationException se avviene una
     * modifica concorrente degli stessi oggetti mediante qualche
     * altro metodo.  
     */
    void remove( );
}
