package lab2;

/**
 * Interfaccia List:
 *
 * Specifica dei metodi per la gestione di una lista
 * unidirezionale. Si noti che esistono altre possibili specifiche di
 * lista con metodi diversi da quelli presentati e usando altri tipi
 * di iteratori.  I metodi qui introdotti si riferiscono alle classi
 * di Java Collections API disponibili in <A HREF=
 * "http://java.sun.com/products/jdk/1.2/docs/guide/collections/reference.html">
 * <TT>java.util.*</TT> (JDK 1.2)</A> [List].
 *
 * <pre>
 * *********************** METODI PUBBLICI ***************************
 * boolean add( Object x )       --> Inserisce x in fondo alla lista
 * boolean remove( Object x )    --> Cancella la prima occorrenza di x
 * boolean contains( Object x )  --> Verifica se x e' nella lista
 * boolean isEmpty( )            --> Verifica se la lista e' vuota
 * void clear( )                 --> Svuota la lista
 * Iterator iterator( )          --> Iteratore per scandire la lista
 * *********************** ECCEZIONI *********************************
 * nessuna generata direttamente
 * </pre>
 * @version 2.0 Febbraio 2001
 * @author A. Brogi      (<A HREF="mailto:brogi@di.unipi.it">brogi@di.unipi.it</A>)
 * @author A. Corradini  (<A HREF="mailto:andrea@di.unipi.it">andrea@di.unipi.it</A>)
 * @author F. Gadducci   (<A HREF="mailto:gadducci@di.unipi.it">gadducci@di.unipi.it</A>)
 * @author R. Grossi     (<A HREF="mailto:grossi@di.unipi.it">grossi@di.unipi.it</A>)
 * @author C. Montangero (<A HREF="mailto:monta@di.unipi.it">monta@di.unipi.it</A>)
 * @author F. Romani     (<A HREF="mailto:romani@di.unipi.it">romani@di.unipi.it</A>)
 * @author L. Semini     (<A HREF="mailto:semini@di.unipi.it">semini@di.unipi.it</A>)
 */
public interface List {
    /**
     * Inserisce un oggetto in fondo alla lista. Non sono ammessi 
     * oggetti null.
     * @param x l'oggetto da inserire, <em>diverso</em> da null.
     * @return  <code>true</code> come richiesto da Java Collection API.
     * @exception IllegalArgumentException se l'argomento passato
     *            &egrave; <code>null</code>.  
     */
    boolean add( Object x );

    /**
     * Cancella la prima occorrenza dell'oggetto a partire dall'inizio
     * della lista, che rimane inalterata se non contiene l'oggetto. 
     * Il metodo standard <tt>equals</tt> viene impiegato per trovare 
     * la prima occorrenza.
     * @param x  l'oggetto da cancellare, se presente e <em>diverso</em> 
     * da null. 
     * @return  <code>true</code> se la lista contiene l'oggetto;
     *          <code>false</code> altrimenti.
     * @exception IllegalArgumentException se l'argomento passato
     *            &egrave; <code>null</code>.  
     */
    boolean remove( Object x ); 
 
    /**
     * Verifica se la lista contiene l'oggetto specificato, usando il
     * metodo standard <tt>equals</tt> (l'oggetto pu&ograve; occorrere
     * pi&ugrave; volte).
     * @param x l'oggetto da cercare nella lista.
     * @return  <code>true</code> se la lista contiene l'oggetto;
     *          <code>false</code> altrimenti.
     * @exception IllegalArgumentException se l'argomento passato
     *            &egrave; <code>null</code>.  
     */
    boolean contains( Object x );

    /**
     * Verifica che la lista sia logicamente vuota.
     * @return  <code>true</code> se la lista &egrave; vuota;
     *          <code>false</code> altrimenti.
     */
    boolean isEmpty( );

    /**
     * Svuota la lista.
     */
    void    clear( );

    /**
     * Restituisce un iteratore sulla lista per scandire gli oggetti
     * in sequenza a partire dall'inizio della lista. Se la lista
     * &egrave; modificata con metodi diversi da quelli
     * dell'iteratore, viene generata una eccezione
     * <tt>ConcurrentModificationException</tt>. In tal caso,
     * l'iterazione termina per evitare comportamenti difficili
     * da prevedere.     
     * @return una classe che realizza <code>Iterator</code> per
     * scandire gli oggetti in sequenza dall'inizio.
     * @exception lab2.Exceptions.ConcurrentModificationException se
     * avviene una modifica concorrente mediante altri metodi.
     * @see java.util.Iterator
     */
    public Iterator iterator( );
}
