package lab2;

import  lab2.Exceptions.*;

/**
 * Interfaccia Queue:
 *
 * Specifica dei metodi per la gestione della coda, per analogia con
 * la classe Stack di Java Collections API disponibile in
 * <code>java.util.*</code>.
 * <pre>
 * ********************* METODI PUBBLICI ***********************
 * Object enqueue( Object x)  --> Inserisce x in fondo alla coda
 * Object dequeue( )          --> Estrae dalla testa della coda
 * Object front( )            --> Restituisce l'elemento in testa
 * boolean isEmpty( )         --> Verifica se la coda e' vuota
 * void clear( )              --> Svuota la coda
 * ******************** ECCEZIONI ******************************
 * dequeue e front generano EmptyQueueException con coda vuota
 * </pre>
 * @version 2.0 Febbraio 2001
 * @author A. Brogi      (<A HREF="mailto:brogi@di.unipi.it">brogi@di.unipi.it</A>)
 * @author A. Corradini  (<A HREF="mailto:andrea@di.unipi.it">andrea@di.unipi.it</A>)
 * @author F. Gadducci   (<A HREF="mailto:gadducci@di.unipi.it">gadducci@di.unipi.it</A>)
 * @author R. Grossi     (<A HREF="mailto:grossi@di.unipi.it">grossi@di.unipi.it</A>)
 * @author C. Montangero (<A HREF="mailto:monta@di.unipi.it">monta@di.unipi.it</A>)
 * @author F. Romani     (<A HREF="mailto:romani@di.unipi.it">romani@di.unipi.it</A>)
 * @author L. Semini     (<A HREF="mailto:semini@di.unipi.it">semini@di.unipi.it</A>)
 */
public interface Queue {
    /**
     * Inserisce un oggetto in fondo alla coda.
     * @param x  l'oggetto da inserire.
     * @return   l'oggetto inserito.
     * @exception IllegalArgumentException se l'argomento passato
     *            &egrave; <code>null</code>.  
     */
    Object enqueue( Object x );

    /**
     * Rimuove e restituisce l'oggetto in testa alla coda.
     * @return   l'oggetto in testa.
     * @exception EmptyQueueException con coda vuota.
     */
    Object dequeue( );

    /**
     * Restituisce l'oggetto in testa alla coda senza estrarlo.
     * @return   l'oggetto in testa.
     * @exception EmptyQueueException con coda vuota.
     */
    Object front( );

    /**
     * Verifica che la coda sia logicamente vuota.
     * @return  <code>true</code> se la coda &egrave; vuota;
     *          <code>false</code> altrimenti.
     */
    boolean isEmpty( );

    /**
     *  Svuota la coda.
     */
    void clear( ); 
}
