package lab2;

/**
 * Interfaccia Set:
 *
 * Specifica dei metodi per la gestione di un insieme di oggetti, non
 * necessariamente ordinato, che differisce dalla lista <tt>List</tt>
 * in quanto non sono ammessi <em>duplicati</em>. Il riferimento
 * &egrave; a Java Collections API, disponibile in <A
 * HREF="http://java.sun.com/products/jdk/1.2/docs/guide/collections/reference.html"> 
 * <TT>java.util.*</TT> (JDK 1.2)</A> [List, Set] .
 *
 * <pre>
 * *********************** METODI PUBBLICI ***************************
 * boolean add( Object x )       --> Aggiunge x all'insieme
 * boolean remove( Object x )    --> Elimina x dall'insieme
 * boolean contains( Object x )  --> Verifica se x e' nell'insieme
 * boolean isEmpty( )            --> Verifica se l'insieme e' vuoto
 * void clear( )                 --> Rende vuoto l'insieme
 * Iterator iterator( )          --> Iteratore per esaminare l'insieme
 * *********************** ECCEZIONI *********************************
 * nessuna
 * </pre>
 * @version 2.0  Gennaio 2001
 * @author A. Brogi      (<A HREF="mailto:brogi@di.unipi.it">brogi@di.unipi.it</A>)
 * @author A. Corradini  (<A HREF="mailto:andrea@di.unipi.it">andrea@di.unipi.it</A>)
 * @author F. Gadducci   (<A HREF="mailto:gadducci@di.unipi.it">gadducci@di.unipi.it</A>)
 * @author R. Grossi     (<A HREF="mailto:grossi@di.unipi.it">grossi@di.unipi.it</A>)
 * @author C. Montangero (<A HREF="mailto:monta@di.unipi.it">monta@di.unipi.it</A>)
 * @author F. Romani     (<A HREF="mailto:romani@di.unipi.it">romani@di.unipi.it</A>)
 * @author L. Semini     (<A HREF="mailto:semini@di.unipi.it">semini@di.unipi.it</A>)
 */
public interface Set {
    /**
     * Aggiunge un oggetto all'insieme. Non sono ammessi 
     * oggetti null e/o duplicati.
     * @param x l'oggetto da inserire, <em>diverso</em> da null.
     * @return  <code>true</code> se l'oggetto non apparteneva gi&agrave;
     *           all'insieme; <code>false</code> altrimenti.
     * @exception IllegalArgumentException se l'argomento passato
     *            &egrave; <code>null</code>.  
     */
    boolean add( Object x );

    /**
     * Elimina un oggetto dall'insieme, che rimane inalterato se non 
     * contiene l'oggetto. Il metodo standard <tt>equals</tt> viene 
     * impiegato per trovare tale oggetto.
     * @param x  l'oggetto da eliminare, se presente e <em>diverso</em> 
     * da null. 
     * @return  <code>true</code> se l'oggetto appartiene all'insieme;
     *          <code>false</code> altrimenti.
     * @exception IllegalArgumentException se l'argomento passato
     *            &egrave; <code>null</code>.  
     */
    boolean remove( Object x ); 
 
    /**
     * Verifica se l'insieme contiene l'oggetto specificato, usando il
     * metodo standard <tt>equals</tt> (l'oggetto pu&ograve; occorrere
     * al pi&ugrave; una volta).
     * @param x l'oggetto da cercare nell'insieme.
     * @return  <code>true</code> se l'insieme contiene l'oggetto;
     *          <code>false</code> altrimenti.
     * @exception IllegalArgumentException se l'argomento passato
     *            &egrave; <code>null</code>.  
     */
    boolean contains( Object x );

    /**
     * Verifica che l'insieme sia logicamente vuoto.
     * @return  <code>true</code> se l'insieme &egrave; vuoto;
     *          <code>false</code> altrimenti.
     */
    boolean isEmpty( );

    /**
     * Rende l'insieme vuoto.
     */
    void    clear( );

    /**
     * Restituisce un iteratore per scandire gli oggetti dell'insieme
     * senza un ordine particolare. Se l'insieme &egrave; mantenuto
     * ordinato, l'iteratore scandisce gli oggetti secondo
     * l'ordine. Nel caso che l'insieme venga modificato con metodi
     * diversi da quelli dell'iteratore, viene generata una eccezione
     * <tt>ConcurrentModificationException</tt>. In questo modo,
     * l'iterazione termina per evitare comportamenti difficili da
     * prevedere.
     * @return una classe che realizza <code>Iterator</code> per
     * scandire gli oggetti dell'insieme.
     * @exception lab2.Exceptions.ConcurrentModificationException se
     * avviene una modifica concorrente mediante altri metodi.
     * @see java.util.Iterator
     */
    public Iterator iterator( );

}
