abstract class Shape
{
  public abstract void print();
  public abstract double area();
}

abstract class TwoDimensionalShape extends Shape
{
  protected Punto2D centro;
  
  public TwoDimensionalShape()
    {
      centro = new Punto2D(0,0);
    }

  public TwoDimensionalShape(Punto2D centro)
    {
      this.centro = centro;
    }

  public abstract double perimetro();

  public void print()
    {
      System.out.println(getClass().getName());
      System.out.println("Centro = "+ centro.toString());
    }
}


abstract class ThreeDimensionalShape extends Shape
{
  protected Punto3D centro;
  
  public ThreeDimensionalShape()
    {
      centro = new Punto3D(0,0,0);
    }

  public ThreeDimensionalShape(Punto3D centro)
    {
      this.centro = centro;
    }

  public void print()
    {
      System.out.println(getClass().getName());
      System.out.println("Centro = "+ centro.toString());
    }

  public abstract double volume();
}


class Quadrato extends TwoDimensionalShape
{
  protected double base;

  public Quadrato()
  {
    super();
    base = 0;
  }

  public Quadrato(Punto2D centro, double base)
  {
    super(centro);
    this.base = base;
  }

  public void print()
  {
    super.print();
    System.out.println("Base = "+ base);
  }

  public double perimetro()
  {
    return 4 * base;
  }

  public double area()
  {
    return base * base;
  }
}

class Rettangolo extends Quadrato
{
  protected double altezza;
  
  public Rettangolo()
  {
    super();
    altezza = 0;
  }

  public Rettangolo(Punto2D centro, double base, double altezza)
  {
    super(centro, base);
    this.altezza = altezza;
  }

  public void print()
  {
    super.print();
    System.out.println("Altezza = "+ altezza);
  }

 public double perimetro()
  {
    return 2 * base + 2 * altezza;
  }

  public double area()
  {
    return base * altezza;
  }
}

class Cerchio extends TwoDimensionalShape
{
  
  protected double raggio;

  public Cerchio()
  {
    super();
    raggio = 0;
  }

  public Cerchio(Punto2D centro, double raggio)
  {
    super(centro);
    this.raggio = raggio;
  }

  public void print()
  {
    super.print();
    System.out.println("Raggio = "+ raggio);
  }

  public double perimetro()
  {
    return 2 * Math.PI * raggio;
  }

  public double area()
  {
    return Math.PI * raggio * raggio;
  }
}

class Cubo extends ThreeDimensionalShape
{
  protected double base;

  public Cubo()
  {
    super();
    base = 0;
  }

  public Cubo(Punto3D centro, double lato)
  {
    super(centro);
    base = lato;
  }

  public void print()
  {
    super.print();
    System.out.println("Lato = "+ base);
  }

  public double volume()
  {
    return Math.pow(base,3);
  }

  public double area()
  {
    return 6 * base * base;
  }
}

class Parallelepipedo extends Cubo
{
  protected double altezza, profondita;

  public Parallelepipedo()
  {
    super();
    altezza = profondita = 0;
  }

  public Parallelepipedo(Punto3D centro, double base, double altezza, double profondita)
  {
    super(centro, base);
    this.altezza = altezza;
    this.profondita = profondita;
  }

  public void print()
  {
    super.print();
    System.out.println("Altezza = "+ altezza);
    System.out.println("Profondita' = "+ profondita);
  }

  public double volume()
  {
    return base * altezza * profondita;
  }

  public double area()
  {
    return 2 * base * altezza + 2 * base * profondita + 2 * altezza * profondita;
  }
}

class Sfera extends ThreeDimensionalShape
{
  protected double raggio;

  public Sfera()
  {
    super();
    raggio = 0;
  }

  public Sfera(Punto3D centro, double r)
  {
    super(centro);
    raggio = r;
  }

  public void print()
  {
    super.print();
    System.out.println("Raggio = "+ raggio);
  }

  public double volume()
  {
    return 4 * Math.PI * Math.pow(raggio,3) / 3;
  }

  public double area()
  {
    return 2 * Math.PI * Math.pow(raggio,2);
  }
}

public class GerarchiaShape
{
  public static void main(String[] args)
  {
    Punto2D origine2D = new Punto2D(0,0);
    TwoDimensionalShape figura = new Quadrato(origine2D, 4);
    figura.print();
    System.out.println("Perimetro = "+figura.perimetro());
    System.out.println("Area = "+figura.area());
    System.out.println();
    figura = new Rettangolo(origine2D, 4, 2);
    figura.print();
    System.out.println("Perimetro = "+figura.perimetro());
    System.out.println("Area = "+figura.area());
    System.out.println();
    figura = new Cerchio(origine2D, 4);
    figura.print();
    System.out.println("Perimetro = "+figura.perimetro());
    System.out.println("Area = "+figura.area());
    System.out.println();
    Punto3D origine3D = new Punto3D(0,0,0);
    ThreeDimensionalShape figura3d = new Cubo(origine3D, 4);
    figura3d.print();
    System.out.println("Area = "+figura3d.area());
    System.out.println("Volume = "+figura3d.volume());
    System.out.println();
    figura3d = new Parallelepipedo(origine3D, 4,2,1);
    figura3d.print();
    System.out.println("Area = "+figura3d.area());
    System.out.println("Volume = "+figura3d.volume());
    System.out.println();
    figura3d = new Sfera(origine3D, 4);
    figura3d.print();
    System.out.println("Area = "+figura3d.area());
    System.out.println("Volume = "+figura3d.volume());
  }
}





