package lab2;

import lab2.Exceptions.*;

/**
 * Classe ListQueue
 *
 * Realizzazione della coda mediante liste
 * @see lab2.ListNode
 * @see lab2.Queue
 */
public class ListQueue implements Queue {
  protected ListNode testa;
  protected ListNode fondo;
  
  /**
   * Costruttore della coda.
   */
  public ListQueue() {
    testa = fondo = null;
  }
  
  /**
   * Inserisce un oggetto in fondo alla coda.
   * @param x  l'oggetto da inserire.
   * @return   l'oggetto inserito.
   * @exception IllegalArgumentException se l'argomento passato
   *            &egrave; <code>null</code>.  
   */
  public Object enqueue( Object x ) {
    if ( x == null )
      throw new IllegalArgumentException( );
    if (isEmpty()) // primo elemento
      testa = fondo = new ListNode(x); 
    else { // elemnti successivi
      fondo.next = new ListNode(x);
      fondo = fondo.next;
    }
    return x;
  }
  
  /**
   * Rimuove e restituisce l'oggetto in testa alla coda.
   * @return   l'oggetto in testa.
   * @exception EmptyQueueException con coda vuota.
   */
  public Object dequeue( ) {
    if ( isEmpty( ) )
      throw new EmptyQueueException( );
    Object item = testa.element;
    if (testa == fondo) // c'e' un solo elemento nella coda
	fondo = null;
    testa = testa.next;
    return item;
  }
  
  /**
   * Restituisce l'oggetto in testa alla coda senza estrarlo.
   * @return   l'oggetto in testa.
   * @exception EmptyQueueException con coda vuota.
   */
  public Object front( ) {
    if ( isEmpty( ) ) 
      throw new EmptyQueueException( );
    return testa.element;
    }
  
  /**
   * Verifica che la coda sia logicamente vuota.
   * @return  <code>true</code> se la coda &egrave; vuota;
   *          <code>false</code> altrimenti.
   */
  public boolean isEmpty( ) {
    return testa == null;
  }
  
  /**
   *  Svuota la coda.
   */
  public void clear( ) {
        testa = fondo = null;
  }
}




